package javax.swing;

import javax.swing.event.MenuKeyEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.MouseMotionAdapter;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusAdapter;
import java.awt.Frame;
import java.awt.Component;
import java.awt.Container;
import javax.swing.event.MenuKeyListener;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.KeyboardFocusManager;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.GraphicsConfiguration;
import java.awt.event.KeyListener;
import java.awt.event.FocusListener;
import java.awt.Rectangle;
import java.awt.Window;
import java.awt.event.MouseEvent;
import java.awt.Point;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseAdapter;

public class ToolTipManager extends MouseAdapter implements MouseMotionListener
{
    Timer enterTimer;
    Timer exitTimer;
    Timer insideTimer;
    String toolTipText;
    Point preferredLocation;
    JComponent insideComponent;
    MouseEvent mouseEvent;
    boolean showImmediately;
    private static final Object TOOL_TIP_MANAGER_KEY;
    transient Popup tipWindow;
    private Window window;
    JToolTip tip;
    private Rectangle popupRect;
    private Rectangle popupFrameRect;
    boolean enabled;
    private boolean tipShowing;
    private FocusListener focusChangeListener;
    private MouseMotionListener moveBeforeEnterListener;
    private KeyListener accessibilityKeyListener;
    private KeyStroke postTip;
    private KeyStroke hideTip;
    protected boolean lightWeightPopupEnabled;
    protected boolean heavyWeightPopupEnabled;
    
    ToolTipManager() {
        this.popupRect = null;
        this.popupFrameRect = null;
        this.enabled = true;
        this.tipShowing = false;
        this.focusChangeListener = null;
        this.moveBeforeEnterListener = null;
        this.accessibilityKeyListener = null;
        this.lightWeightPopupEnabled = true;
        this.heavyWeightPopupEnabled = false;
        (this.enterTimer = new Timer(750, new ToolTipManager.insideTimerAction())).setRepeats(false);
        (this.exitTimer = new Timer(500, new ToolTipManager.outsideTimerAction())).setRepeats(false);
        (this.insideTimer = new Timer(4000, new ToolTipManager.stillInsideTimerAction())).setRepeats(false);
        this.moveBeforeEnterListener = new ToolTipManager.MoveBeforeEnterListener();
        this.accessibilityKeyListener = new ToolTipManager.AccessibilityKeyListener();
        this.postTip = KeyStroke.getKeyStroke(112, 2);
        this.hideTip = KeyStroke.getKeyStroke(27, 0);
    }
    
    public void setEnabled(boolean enabled) {
        if (!(this.enabled = enabled)) {
            this.hideTipWindow();
        }
    }
    
    public boolean isEnabled() {
        return this.enabled;
    }
    
    public void setLightWeightPopupEnabled(boolean lightWeightPopupEnabled) {
        this.lightWeightPopupEnabled = lightWeightPopupEnabled;
    }
    
    public boolean isLightWeightPopupEnabled() {
        return this.lightWeightPopupEnabled;
    }
    
    public void setInitialDelay(int initialDelay) {
        this.enterTimer.setInitialDelay(initialDelay);
    }
    
    public int getInitialDelay() {
        return this.enterTimer.getInitialDelay();
    }
    
    public void setDismissDelay(int initialDelay) {
        this.insideTimer.setInitialDelay(initialDelay);
    }
    
    public int getDismissDelay() {
        return this.insideTimer.getInitialDelay();
    }
    
    public void setReshowDelay(int initialDelay) {
        this.exitTimer.setInitialDelay(initialDelay);
    }
    
    public int getReshowDelay() {
        return this.exitTimer.getInitialDelay();
    }
    
    private GraphicsConfiguration getDrawingGC(Point p) {
        GraphicsDevice[] screenDevices = GraphicsEnvironment.getLocalGraphicsEnvironment().getScreenDevices();
        for (int length = screenDevices.length, i = 0; i < length; ++i) {
            GraphicsConfiguration[] configurations = screenDevices[i].getConfigurations();
            int length2 = configurations.length;
            int j = 0;
            while (j < length2) {
                GraphicsConfiguration graphicsConfiguration = configurations[j];
                if (graphicsConfiguration.getBounds().contains(p)) {
                    return graphicsConfiguration;
                }
                ++j;
            }
        }
        return null;
    }
    
    void showTipWindow() {
        if (this.insideComponent == null || !this.insideComponent.isShowing()) {
            return;
        }
        if ("activeApplication".equals(UIManager.getString("ToolTipManager.enableToolTipMode")) && KeyboardFocusManager.getCurrentKeyboardFocusManager().getFocusedWindow() == null) {
            return;
        }
        if (this.enabled) {
            Point locationOnScreen = this.insideComponent.getLocationOnScreen();
            Point point;
            if (this.preferredLocation != null) {
                point = new Point(locationOnScreen.x + this.preferredLocation.x, locationOnScreen.y + this.preferredLocation.y);
            }
            else {
                point = this.mouseEvent.getLocationOnScreen();
            }
            GraphicsConfiguration gc = this.getDrawingGC(point);
            if (gc == null) {
                point = this.mouseEvent.getLocationOnScreen();
                gc = this.getDrawingGC(point);
                if (gc == null) {
                    gc = this.insideComponent.getGraphicsConfiguration();
                }
            }
            Rectangle bounds = gc.getBounds();
            Insets screenInsets = Toolkit.getDefaultToolkit().getScreenInsets(gc);
            bounds.x += screenInsets.left;
            bounds.y += screenInsets.top;
            bounds.width -= screenInsets.left + screenInsets.right;
            bounds.height -= screenInsets.top + screenInsets.bottom;
            int leftToRight = SwingUtilities.isLeftToRight(this.insideComponent);
            this.hideTipWindow();
            (this.tip = this.insideComponent.createToolTip()).setTipText(this.toolTipText);
            Dimension preferredSize = this.tip.getPreferredSize();
            if (this.preferredLocation != null) {
                if (leftToRight == 0) {
                    point.x -= preferredSize.width;
                }
            }
            else {
                point = new Point(locationOnScreen.x + this.mouseEvent.getX(), locationOnScreen.y + this.mouseEvent.getY() + 20);
                if (leftToRight == 0 && point.x - preferredSize.width >= 0) {
                    point.x -= preferredSize.width;
                }
            }
            if (this.popupRect == null) {
                this.popupRect = new Rectangle();
            }
            this.popupRect.setBounds(point.x, point.y, preferredSize.width, preferredSize.height);
            if (point.x < bounds.x) {
                point.x = bounds.x;
            }
            else if (point.x - bounds.x + preferredSize.width > bounds.width) {
                point.x = bounds.x + Math.max(0, bounds.width - preferredSize.width);
            }
            if (point.y < bounds.y) {
                point.y = bounds.y;
            }
            else if (point.y - bounds.y + preferredSize.height > bounds.height) {
                point.y = bounds.y + Math.max(0, bounds.height - preferredSize.height);
            }
            PopupFactory sharedInstance = PopupFactory.getSharedInstance();
            if (this.lightWeightPopupEnabled) {
                int popupFitHeight = this.getPopupFitHeight(this.popupRect, this.insideComponent);
                if (this.getPopupFitWidth(this.popupRect, this.insideComponent) > 0 || popupFitHeight > 0) {
                    sharedInstance.setPopupType(1);
                }
                else {
                    sharedInstance.setPopupType(0);
                }
            }
            else {
                sharedInstance.setPopupType(1);
            }
            this.tipWindow = sharedInstance.getPopup(this.insideComponent, this.tip, point.x, point.y);
            sharedInstance.setPopupType(0);
            this.tipWindow.show();
            Window windowForComponent = SwingUtilities.windowForComponent(this.insideComponent);
            this.window = SwingUtilities.windowForComponent(this.tip);
            if (this.window != null && this.window != windowForComponent) {
                this.window.addMouseListener(this);
            }
            else {
                this.window = null;
            }
            this.insideTimer.start();
            this.tipShowing = true;
        }
        return;
    }
    
    void hideTipWindow() {
        if (this.tipWindow != null) {
            if (this.window != null) {
                this.window.removeMouseListener(this);
                this.window = null;
            }
            this.tipWindow.hide();
            this.tipWindow = null;
            this.tipShowing = false;
            this.tip = null;
            this.insideTimer.stop();
        }
    }
    
    public static ToolTipManager sharedInstance() {
        Object appContextGet = SwingUtilities.appContextGet(ToolTipManager.TOOL_TIP_MANAGER_KEY);
        if ((appContextGet instanceof ToolTipManager)) {
            return (ToolTipManager)appContextGet;
        }
        ToolTipManager toolTipManager = new ToolTipManager();
        SwingUtilities.appContextPut(ToolTipManager.TOOL_TIP_MANAGER_KEY, toolTipManager);
        return toolTipManager;
    }
    
    public void registerComponent(JComponent component) {
        component.removeMouseListener(this);
        component.addMouseListener(this);
        component.removeMouseMotionListener(this.moveBeforeEnterListener);
        component.addMouseMotionListener(this.moveBeforeEnterListener);
        if ((component instanceof JMenuItem)) {
            ((JMenuItem)component).removeMenuKeyListener((MenuKeyListener)this.accessibilityKeyListener);
            ((JMenuItem)component).addMenuKeyListener((MenuKeyListener)this.accessibilityKeyListener);
        }
        else {
            component.removeKeyListener(this.accessibilityKeyListener);
            component.addKeyListener(this.accessibilityKeyListener);
        }
    }
    
    public void unregisterComponent(JComponent component) {
        component.removeMouseListener(this);
        component.removeMouseMotionListener(this.moveBeforeEnterListener);
        if ((component instanceof JMenuItem)) {
            ((JMenuItem)component).removeMenuKeyListener((MenuKeyListener)this.accessibilityKeyListener);
        }
        else {
            component.removeKeyListener(this.accessibilityKeyListener);
        }
    }
    
    @Override
    public void mouseEntered(MouseEvent mouseEvent) {
        this.initiateToolTip(mouseEvent);
    }
    
    private void initiateToolTip(MouseEvent mouseEvent) {
        if (mouseEvent.getSource() == this.window) {
            return;
        }
        JComponent insideComponent = (JComponent)mouseEvent.getSource();
        insideComponent.removeMouseMotionListener(this.moveBeforeEnterListener);
        this.exitTimer.stop();
        Point point = mouseEvent.getPoint();
        if (point.x < 0 || point.x >= insideComponent.getWidth() || point.y < 0 || point.y >= insideComponent.getHeight()) {
            return;
        }
        if (this.insideComponent != null) {
            this.enterTimer.stop();
        }
        insideComponent.removeMouseMotionListener(this);
        insideComponent.addMouseMotionListener(this);
        int n = this.insideComponent == insideComponent;
        this.insideComponent = insideComponent;
        if (this.tipWindow != null) {
            this.mouseEvent = mouseEvent;
            if (this.showImmediately) {
                String toolTipText = insideComponent.getToolTipText(mouseEvent);
                Point toolTipLocation = insideComponent.getToolTipLocation(mouseEvent);
                int n2 = (this.preferredLocation != null) ? this.preferredLocation.equals(toolTipLocation) : (toolTipLocation == null);
                if (n == 0 || !this.toolTipText.equals(toolTipText) || n2 == 0) {
                    this.toolTipText = toolTipText;
                    this.preferredLocation = toolTipLocation;
                    this.showTipWindow();
                }
            }
            else {
                this.enterTimer.start();
            }
        }
        return;
    }
    
    @Override
    public void mouseExited(MouseEvent mouseEvent) {
        boolean n = true;
        JComponent insideComponent = this.insideComponent;
        if (this.window != null && mouseEvent.getSource() == this.window && this.insideComponent != null) {
            Container topLevelAncestor = this.insideComponent.getTopLevelAncestor();
            if (topLevelAncestor != null) {
                Point point2 = mouseEvent.getPoint();
                SwingUtilities.convertPointToScreen(point2, this.window);
                point2.x -= topLevelAncestor.getX();
                point2.y -= topLevelAncestor.getY();
                Point convertPoint2 = SwingUtilities.convertPoint(null, point2, this.insideComponent);
                n = (convertPoint2.x < 0 || convertPoint2.x >= this.insideComponent.getWidth() || convertPoint2.y < 0 || convertPoint2.y >= this.insideComponent.getHeight());
            }
        }
        else if (mouseEvent.getSource() == this.insideComponent && this.tipWindow != null) {
            Window windowAncestor = SwingUtilities.getWindowAncestor(this.insideComponent);
            if (windowAncestor != null) {
                Point convertPoint = SwingUtilities.convertPoint(this.insideComponent, mouseEvent.getPoint(), windowAncestor);
                Rectangle bounds = this.insideComponent.getTopLevelAncestor().getBounds();
                convertPoint.x += bounds.x;
                convertPoint.y += bounds.y;
                Point point = new Point(0, 0);
                SwingUtilities.convertPointToScreen(point, this.tip);
                bounds.x = point.x;
                bounds.y = point.y;
                bounds.width = this.tip.getWidth();
                bounds.height = this.tip.getHeight();
                n = (convertPoint.x < bounds.x || convertPoint.x >= bounds.x + bounds.width || convertPoint.y < bounds.y || convertPoint.y >= bounds.y + bounds.height);
            }
        }
        if (!(n == 0)) {
            this.enterTimer.stop();
            if (this.insideComponent != null) {
                this.insideComponent.removeMouseMotionListener(this);
            }
            this.insideComponent = null;
            this.toolTipText = null;
            this.mouseEvent = null;
            this.hideTipWindow();
            this.exitTimer.restart();
        }
    }
    
    @Override
    public void mousePressed(MouseEvent mouseEvent) {
        this.hideTipWindow();
        this.enterTimer.stop();
        this.showImmediately = false;
        this.insideComponent = null;
        this.mouseEvent = null;
    }
    
    @Override
    public void mouseDragged(MouseEvent mouseEvent) {
    }
    
    @Override
    public void mouseMoved(MouseEvent mouseEvent) {
        if (this.tipShowing) {
            this.checkForTipChange(mouseEvent);
        }
        else if (this.showImmediately) {
            JComponent insideComponent = (JComponent)mouseEvent.getSource();
            this.toolTipText = insideComponent.getToolTipText(mouseEvent);
            if (this.toolTipText != null) {
                this.preferredLocation = insideComponent.getToolTipLocation(mouseEvent);
                this.mouseEvent = mouseEvent;
                this.insideComponent = insideComponent;
                this.exitTimer.stop();
                this.showTipWindow();
            }
        }
        else {
            this.insideComponent = (JComponent)mouseEvent.getSource();
            this.mouseEvent = mouseEvent;
            this.toolTipText = null;
            this.enterTimer.restart();
        }
    }
    
    private void checkForTipChange(MouseEvent mouseEvent) {
        JComponent component = (JComponent)mouseEvent.getSource();
        String toolTipText = component.getToolTipText(mouseEvent);
        Point toolTipLocation = component.getToolTipLocation(mouseEvent);
        if (toolTipText != null || toolTipLocation != null) {
            this.mouseEvent = mouseEvent;
            if (((toolTipText != null && toolTipText.equals(this.toolTipText)) || toolTipText == null) && ((toolTipLocation != null && toolTipLocation.equals(this.preferredLocation)) || toolTipLocation == null)) {
                if (this.tipWindow != null) {
                    this.insideTimer.restart();
                }
                else {
                    this.enterTimer.restart();
                }
            }
            else {
                this.toolTipText = toolTipText;
                this.preferredLocation = toolTipLocation;
                if (this.showImmediately) {
                    this.hideTipWindow();
                    this.showTipWindow();
                    this.exitTimer.stop();
                }
                else {
                    this.enterTimer.restart();
                }
            }
        }
        else {
            this.toolTipText = null;
            this.preferredLocation = null;
            this.mouseEvent = null;
            this.insideComponent = null;
            this.hideTipWindow();
            this.enterTimer.stop();
            this.exitTimer.restart();
        }
    }
    
    static Frame frameForComponent(Component parent) {
        while (!(parent instanceof Frame)) {
            parent = parent.getParent();
        }
        return (Frame)parent;
    }
    
    private FocusListener createFocusChangeListener() {
        return new FocusAdapter() {
            @Override
            public void focusLost(FocusEvent focusEvent) {
                ToolTipManager.this.hideTipWindow();
                ToolTipManager.this.insideComponent = null;
                ((JComponent)focusEvent.getSource()).removeFocusListener(ToolTipManager.this.focusChangeListener);
            }
        };
    }
    
    private int getPopupFitWidth(Rectangle rectangle, Component component) {
        if (component != null) {
            Container container = component.getParent();
            while (container != null) {
                if (container instanceof JFrame || container instanceof JDialog || container instanceof JWindow) {
                    return this.getWidthAdjust(container.getBounds(), rectangle);
                }
                if (container instanceof JApplet || container instanceof JInternalFrame) {
                    if (this.popupFrameRect == null) {
                        this.popupFrameRect = new Rectangle();
                    }
                    Point locationOnScreen = container.getLocationOnScreen();
                    this.popupFrameRect.setBounds(locationOnScreen.x, locationOnScreen.y, container.getBounds().width, container.getBounds().height);
                    return this.getWidthAdjust(this.popupFrameRect, rectangle);
                }
                else {
                    container = container.getParent();
                }
            }
        }
        return 0;
    }
    
    private int getPopupFitHeight(Rectangle rectangle, Component component) {
        if (component != null) {
            Container container = component.getParent();
            while (container != null) {
                if (container instanceof JFrame || container instanceof JDialog || container instanceof JWindow) {
                    return this.getHeightAdjust(container.getBounds(), rectangle);
                }
                if (container instanceof JApplet || container instanceof JInternalFrame) {
                    if (this.popupFrameRect == null) {
                        this.popupFrameRect = new Rectangle();
                    }
                    Point locationOnScreen = container.getLocationOnScreen();
                    this.popupFrameRect.setBounds(locationOnScreen.x, locationOnScreen.y, container.getBounds().width, container.getBounds().height);
                    return this.getHeightAdjust(this.popupFrameRect, rectangle);
                }
                else {
                    container = container.getParent();
                }
            }
        }
        return 0;
    }
    
    private int getHeightAdjust(Rectangle rectangle, Rectangle rectangle2) {
        if (rectangle2.y >= rectangle.y && rectangle2.y + rectangle2.height <= rectangle.y + rectangle.height) {
            return 0;
        }
        return rectangle2.y + rectangle2.height - (rectangle.y + rectangle.height) + 5;
    }
    
    private int getWidthAdjust(Rectangle rectangle, Rectangle rectangle2) {
        if (rectangle2.x >= rectangle.x && rectangle2.x + rectangle2.width <= rectangle.x + rectangle.width) {
            return 0;
        }
        return rectangle2.x + rectangle2.width - (rectangle.x + rectangle.width) + 5;
    }
    
    private void show(JComponent insideComponent) {
        if (this.tipWindow != null) {
            this.hideTipWindow();
            this.insideComponent = null;
        }
        else {
            this.hideTipWindow();
            this.enterTimer.stop();
            this.exitTimer.stop();
            this.insideTimer.stop();
            this.insideComponent = insideComponent;
            if (this.insideComponent != null) {
                this.toolTipText = this.insideComponent.getToolTipText();
                this.preferredLocation = new Point(10, this.insideComponent.getHeight() + 10);
                this.showTipWindow();
                if (this.focusChangeListener == null) {
                    this.focusChangeListener = this.createFocusChangeListener();
                }
                this.insideComponent.addFocusListener(this.focusChangeListener);
            }
        }
    }
    
    private void hide(JComponent component) {
        this.hideTipWindow();
        component.removeFocusListener(this.focusChangeListener);
        this.preferredLocation = null;
        this.insideComponent = null;
    }
    
    static void access$200(ToolTipManager toolTipManager, MouseEvent mouseEvent) {
        toolTipManager.initiateToolTip(mouseEvent);
    }
    
    static void access$500(ToolTipManager toolTipManager, JComponent component) {
        toolTipManager.hide(component);
    }
    
    static void access$700(ToolTipManager toolTipManager, JComponent component) {
        toolTipManager.show(component);
    }
    
    static {
        TOOL_TIP_MANAGER_KEY = new Object();
    }
    
    protected class insideTimerAction implements ActionListener
    {
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            if (ToolTipManager.this.insideComponent != null && ToolTipManager.this.insideComponent.isShowing()) {
                if (ToolTipManager.this.toolTipText == null && ToolTipManager.this.mouseEvent != null) {
                    ToolTipManager.this.toolTipText = ToolTipManager.this.insideComponent.getToolTipText(ToolTipManager.this.mouseEvent);
                    ToolTipManager.this.preferredLocation = ToolTipManager.this.insideComponent.getToolTipLocation(ToolTipManager.this.mouseEvent);
                }
                if (ToolTipManager.this.toolTipText != null) {
                    ToolTipManager.this.showImmediately = true;
                    ToolTipManager.this.showTipWindow();
                }
                else {
                    ToolTipManager.this.insideComponent = null;
                    ToolTipManager.this.toolTipText = null;
                    ToolTipManager.this.preferredLocation = null;
                    ToolTipManager.this.mouseEvent = null;
                    ToolTipManager.this.hideTipWindow();
                }
            }
        }
    }
    
    protected class outsideTimerAction implements ActionListener
    {
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            ToolTipManager.this.showImmediately = false;
        }
    }
    
    protected class stillInsideTimerAction implements ActionListener
    {
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            ToolTipManager.this.hideTipWindow();
            ToolTipManager.this.enterTimer.stop();
            ToolTipManager.this.showImmediately = false;
            ToolTipManager.this.insideComponent = null;
            ToolTipManager.this.mouseEvent = null;
        }
    }
    
    private class MoveBeforeEnterListener extends MouseMotionAdapter
    {
        @Override
        public void mouseMoved(MouseEvent mouseEvent) {
            ToolTipManager.access$200(ToolTipManager.this, mouseEvent);
        }
    }
    
    private class AccessibilityKeyListener extends KeyAdapter implements MenuKeyListener
    {
        @Override
        public void keyPressed(KeyEvent keyEvent) {
            if (!keyEvent.isConsumed()) {
                JComponent component = (JComponent)keyEvent.getComponent();
                KeyStroke keyStrokeForEvent = KeyStroke.getKeyStrokeForEvent(keyEvent);
                if (ToolTipManager.this.hideTip.equals(keyStrokeForEvent)) {
                    if (ToolTipManager.this.tipWindow != null) {
                        ToolTipManager.access$500(ToolTipManager.this, component);
                        keyEvent.consume();
                    }
                }
                else if (ToolTipManager.this.postTip.equals(keyStrokeForEvent)) {
                    ToolTipManager.access$700(ToolTipManager.this, component);
                    keyEvent.consume();
                }
            }
        }
        
        @Override
        public void menuKeyTyped(MenuKeyEvent menuKeyEvent) {
        }
        
        @Override
        public void menuKeyPressed(MenuKeyEvent menuKeyEvent) {
            if (ToolTipManager.this.postTip.equals(KeyStroke.getKeyStrokeForEvent(menuKeyEvent))) {
                MenuElement[] path = menuKeyEvent.getPath();
                MenuElement menuElement = path[path.length - 1];
                MenuElement[] selectedPath = menuKeyEvent.getMenuSelectionManager().getSelectedPath();
                if (menuElement.equals(selectedPath[selectedPath.length - 1])) {
                    ToolTipManager.access$700(ToolTipManager.this, (JComponent)menuElement.getComponent());
                    menuKeyEvent.consume();
                }
            }
        }
        
        @Override
        public void menuKeyReleased(MenuKeyEvent menuKeyEvent) {
        }
    }
}
